class Asset(object):
    def __init__(self, name, geometry, properties):
        self.name = name
        self.geometry = geometry
        self.properties = properties

    @property
    def __geo_interface__(self):
        return {
              "type": "Feature",
              "properties": self.properties,
              "geometry": self.geometry
            }


class ParseError(Exception):
    pass
    
class DataSource(object):

    def __init__(self):
        raise NotImplemented

    def parse(self):
        try:
            result = self._parse()
        except ParseError:
            raise
        except Exception as e:
            raise ParseError(e)

        return result


    def _parse(self):

        '''Must return a (crs, data) tuple.
        crs must be
        recognizable by fiona.crs
        data must be a dictionary of Asset objects,
        keyed by asset ID
        '''
        raise NotImplemented('Must be implemented in a sub class')


class GenericDataSource(DataSource):
    def __init__(self, parser):
        self.parser = parser

    def _parse(self):
        return self.parser()


    

class ShapeFile(DataSource):

    def __init__(self, name, path, id_field, name_field):
        ''''''
        self.name = name
        self.path = path
        self.id_field = id_field
        self.name_field = name_field




    def _parse(self):

        import fiona
        data = {}
        with fiona.open(self.path) as c:
            crs = c.crs
            for rec in c:
                if self.id_field not in rec['properties']:
                    raise ParseError

                a = Asset(rec['properties'].get(self.name_field),
                          rec['geometry'],
                          rec['properties'])
                ID = rec['properties'][self.id_field]

                data[ID] = a
            
        return crs, data
